codes ={
    200: 'success',
    201: 'Auth failed',
    202: 'Missing Manditory Params. See data object for missing params in "records"',
    203: 'Controller disabled',
    204: 'Scheduling failed',
    205: 'Password Mismatch',
    206: 'Oops Something Went Wrong. Try a new user name.',
    207: 'Username does not exist.',
    208: 'Invalid Param Type Cast',
    209: 'Values in submission must be unique',
    210: 'Invalid identifiers',
    290: 'Api Error',
    291: 'Api server message format error : missing param method',
    292: 'Api server message format error : unregistered method',
    293: 'Api server method error',
}


class Response:
    def __init__(self, res =None):
        self.res = dict({'success': True, 'data': {}, 'response_code': 200, 'msg': 'success'})

        if isinstance(res, Response):
            self.data(Response.msg['data'])
            tmp = Response
            tmp.pop('data')
            self.res = dict(**self.res, **tmp)

    # def __repr__(self):
    #     return repr(self.res)

    def fail(self, err_code):
        self.res['success'] = False
        self.res['response_code'] = err_code
        self.res['msg'] = codes[err_code]
        return self
    def success(self):
        return self.res['success']
    def data(self, data):
        if isinstance(data, list):
            self.res['data'] = dict(**self.res['data'], **{'records': data})
            return self

        if not isinstance(data, dict):
            raise TypeError('data object must be of type dict or list')
            return

        self.res['data'] = dict(**self.res['data'], **data)
        return self

    def msg(self):
        return self.res






from functools import wraps
from flask import Flask, request, jsonify, redirect, session
from salutake import msg
import numpy as np
import pprint as pp

def validateParams(manditory,req):

    return all(k in req for k in manditory)


def Auth():

    if 'auth' in session:
        return int(session['auth'])
    return False


def requestVars( manditory=[], authLevel=1):
    def decorator(f):
        @wraps(f)
        def wrapper(*args, **kwargs):
            ath = Auth()
            if not ath or ath < authLevel:
                return redirect('/')
            if request.is_json:
                p = request.get_json(force=True)
            else:
                p = request.args.to_dict()
            if len(manditory):
                if not validateParams(manditory, p):
                    return jsonify(msg.Response().fail(202).data(np.setdiff1d(manditory, list(p.keys())).tolist()).msg())
            if 'symbol' in p:
                p['symbol'] = str(p['symbol']).lower()
            return f(p,)
        return wrapper
    return decorator


def apiAuth(db, manditory=[], authLevel=1):

    def decorator(f):
        @wraps(f)
        def wrapper(*args, **kwargs):

            if request.is_json:
                p = request.get_json()
            else:
                p = request.args.to_dict()

            if len(manditory):
                if not validateParams(manditory, p):
                    return jsonify(msg.Response().fail(202).data(np.setdiff1d(manditory, p).tolist()).msg())
            #if not choke.authenticate(db, p['username'], p['password']):
                #pass
                #return jsonify(msg.Response().fail(201).msg())

            return f(p,)
        return wrapper
    return decorator